# API Docs - v1.0.2

## Sinkmapper

### avro *<a target="_blank" href="https://wso2.github.io/siddhi/documentation/siddhi-4.0/#sink-mapper">(Sink Mapper)</a>*

<p style="word-wrap: break-word">This extension is a Siddhi Event to Avro Message output mapper.Transports that publish  messages to Avro sink can utilize this extension to convert siddhi events to Avro messages.<br>&nbsp;Users can either specify the avro schema or give the schema registry URL and schema reference id as a parameter in stream definition.<br>In case no specification of avro schema a flat avro schema of type record is generated using the stream attributes as schema fields.</p>

<span id="syntax" class="md-typeset" style="display: block; font-weight: bold;">Syntax</span>
```
@sink(..., @map(type="avro", schema.def="<STRING>", schema.registry="<STRING>", schema.id="<STRING>")
```

<span id="query-parameters" class="md-typeset" style="display: block; color: rgba(0, 0, 0, 0.54); font-size: 12.8px; font-weight: bold;">QUERY PARAMETERS</span>
<table>
    <tr>
        <th>Name</th>
        <th style="min-width: 20em">Description</th>
        <th>Default Value</th>
        <th>Possible Data Types</th>
        <th>Optional</th>
        <th>Dynamic</th>
    </tr>
    <tr>
        <td style="vertical-align: top">schema.def</td>
        <td style="vertical-align: top; word-wrap: break-word">This specifies the desired avro schema to be used to convert siddhi events to avro message.<br>The schema should be specified as a quoted json string.</td>
        <td style="vertical-align: top"></td>
        <td style="vertical-align: top">STRING</td>
        <td style="vertical-align: top">No</td>
        <td style="vertical-align: top">No</td>
    </tr>
    <tr>
        <td style="vertical-align: top">schema.registry</td>
        <td style="vertical-align: top; word-wrap: break-word">Used to specify the URL of the schema registry.</td>
        <td style="vertical-align: top"></td>
        <td style="vertical-align: top">STRING</td>
        <td style="vertical-align: top">No</td>
        <td style="vertical-align: top">No</td>
    </tr>
    <tr>
        <td style="vertical-align: top">schema.id</td>
        <td style="vertical-align: top; word-wrap: break-word">Used to specify the id of the avro schema. This id is the global id returned from the schema registry when posting the schema to the registry. The specified id is used to retrive the schema from the schema registry.</td>
        <td style="vertical-align: top"></td>
        <td style="vertical-align: top">STRING</td>
        <td style="vertical-align: top">No</td>
        <td style="vertical-align: top">No</td>
    </tr>
</table>

<span id="examples" class="md-typeset" style="display: block; font-weight: bold;">Examples</span>
<span id="example-1" class="md-typeset" style="display: block; color: rgba(0, 0, 0, 0.54); font-size: 12.8px; font-weight: bold;">EXAMPLE 1</span>
```
@sink(type='inMemory', topic='stock', @map(type='avro',schema.def = """{"type":"record","name":"stock","namespace":"stock.example","fields":[{"name":"symbol","type":"string"},{"name":"price":"type":"float"},{"name":"volume","type":"long"}]}"""))
define stream stockStream (symbol string, price float, volume long);
```
<p style="word-wrap: break-word">The above configuration performs a default Avro mapping that generates an Avro message as output byte array.</p>

<span id="example-2" class="md-typeset" style="display: block; color: rgba(0, 0, 0, 0.54); font-size: 12.8px; font-weight: bold;">EXAMPLE 2</span>
```
@sink(type='inMemory', topic='stock', @map(type='avro',schema.registry = 'http://localhost:8081', schema.id ='22',@payload("""{"Symbol":{{symbol}},"Price":{{price}},"Volume":{{volume}}}"""
)))
define stream stockStream (symbol string, price float, volume long);
```
<p style="word-wrap: break-word">The above configuration performs a custom Avro mapping that generates an Avro message as output byte array. The avro schema is retrieved from the given schema registry(localhost:8081) using the provided schema id.</p>

## Sourcemapper

### avro *<a target="_blank" href="https://wso2.github.io/siddhi/documentation/siddhi-4.0/#source-mapper">(Source Mapper)</a>*

<p style="word-wrap: break-word">Avro to Event input mapper. Transports which accepts Avro messages can utilize this extension to convert the incoming Avro message to Siddhi event.<br>Users can specify the avro schema used to create avro message as a parameter in stream definition.<br>In case no specification of avro schema a flat avro schema of type record is generated using the stream attributes as schema fields.<br>The generated/specified avro schema is used to convert the avro message into siddhi event.</p>

<span id="syntax" class="md-typeset" style="display: block; font-weight: bold;">Syntax</span>
```
@source(..., @map(type="avro", schema.def="<STRING>", schema.registry="<STRING>", schema.id="<STRING>", fail.on.missing.attribute="<BOOL>")
```

<span id="query-parameters" class="md-typeset" style="display: block; color: rgba(0, 0, 0, 0.54); font-size: 12.8px; font-weight: bold;">QUERY PARAMETERS</span>
<table>
    <tr>
        <th>Name</th>
        <th style="min-width: 20em">Description</th>
        <th>Default Value</th>
        <th>Possible Data Types</th>
        <th>Optional</th>
        <th>Dynamic</th>
    </tr>
    <tr>
        <td style="vertical-align: top">schema.def</td>
        <td style="vertical-align: top; word-wrap: break-word">Used to specify the schema of the Avro message. The full schema used to create the avro message should be specified as quoted json string.</td>
        <td style="vertical-align: top"></td>
        <td style="vertical-align: top">STRING</td>
        <td style="vertical-align: top">No</td>
        <td style="vertical-align: top">No</td>
    </tr>
    <tr>
        <td style="vertical-align: top">schema.registry</td>
        <td style="vertical-align: top; word-wrap: break-word">Used to specify the URL of the schema registry.</td>
        <td style="vertical-align: top"></td>
        <td style="vertical-align: top">STRING</td>
        <td style="vertical-align: top">No</td>
        <td style="vertical-align: top">No</td>
    </tr>
    <tr>
        <td style="vertical-align: top">schema.id</td>
        <td style="vertical-align: top; word-wrap: break-word">Used to specify the id of the avro schema. This id is the global id returned from the schema registry when posting the schema to the registry. The specified id is used to retrive the schema from the schema registry.</td>
        <td style="vertical-align: top"></td>
        <td style="vertical-align: top">STRING</td>
        <td style="vertical-align: top">No</td>
        <td style="vertical-align: top">No</td>
    </tr>
    <tr>
        <td style="vertical-align: top">fail.on.missing.attribute</td>
        <td style="vertical-align: top; word-wrap: break-word">This can either have value true or false. By default it will be true. <br>This attribute allows user to handle unknown attributes.<br>&nbsp;By default if an json execution fails or returns null system will drop that message.<br>However setting this property to false will prompt system to send event with null value to Siddhi where user can handle it accordingly.<br>(ie. Assign a default value)</td>
        <td style="vertical-align: top">true</td>
        <td style="vertical-align: top">BOOL</td>
        <td style="vertical-align: top">Yes</td>
        <td style="vertical-align: top">No</td>
    </tr>
</table>

<span id="examples" class="md-typeset" style="display: block; font-weight: bold;">Examples</span>
<span id="example-1" class="md-typeset" style="display: block; color: rgba(0, 0, 0, 0.54); font-size: 12.8px; font-weight: bold;">EXAMPLE 1</span>
```
@source(type='inMemory', topic='user', @map(type='avro', schema .def = """{"type":"record","name":"userInfo","namespace":"user.example","fields":[{"name":"name","type":"string"}, {"name":"age","type":"int"}]}"""))
define stream userStream (name string, age int );

```
<p style="word-wrap: break-word">Above configuration will do a default Avro input mapping. The input avro message containing user info will be converted to a siddhi event.<br>Expected input is a byte array.</p>

<span id="example-2" class="md-typeset" style="display: block; color: rgba(0, 0, 0, 0.54); font-size: 12.8px; font-weight: bold;">EXAMPLE 2</span>
```
@source(type='inMemory', topic='user', @map(type='avro', schema .def = """{"type":"record","name":"userInfo","namespace":"avro.userInfo","fields":[{"name":"username","type":"string"}, {"name":"age","type":"int"}]}""",@attributes(name="username",age="age")))
define stream userStream (name string, age int );

```
<p style="word-wrap: break-word">Above configuration will do a custom Avro input mapping. The input avro message containing user info will be converted  to a siddhi event.<br>&nbsp;Expected input is a byte array.</p>

<span id="example-3" class="md-typeset" style="display: block; color: rgba(0, 0, 0, 0.54); font-size: 12.8px; font-weight: bold;">EXAMPLE 3</span>
```
@source(type='inMemory', topic='user', @map(type='avro',schema.registry='http://192.168.2.5:9090', schema.id='1',@attributes(name="username",age="age")))
define stream userStream (name string, age int );

```
<p style="word-wrap: break-word">Above configuration will do a custom Avro input mapping. The input avro message containing user info will be converted to a siddhi event using the schema retrived from given schema registry(localhost:8081).<br>Expected input is a byte array.</p>

